/*
 * NVInfo.c - Simple interface to NVCtrl for getting GPU information
 *            on python.
 *
 *  Copyright (C) 2008, Angelo Arrifano <miknix@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  TODO: NVCtrl exports some functions for notification when attributes
 *  are changed. We could use them.
 */

#include <Python.h>
#include <X11/Xlib.h>
#include <NVCtrl/NVCtrlLib.h>


Display *NVInfoDisplay = NULL;

PyObject *Get_Temp(PyObject *pSelf, PyObject *pArgs);
PyObject *Get_Freq(PyObject *pSelf, PyObject *pArgs);

PyMethodDef NVInfoMethods[] = {
	{"Get_Temp", Get_Temp, METH_NOARGS, "Get GPU Temperature."},
	{"Get_Freq", Get_Freq, METH_NOARGS, "Get (GPUCLOCK,MEMCLOCK) Tuple."},
	{NULL, NULL, 0, NULL},
};

PyObject *Get_Temp(PyObject *pSelf, PyObject *pArgs)
{
	int Temp;

	if (!NVInfoDisplay) {
		PyErr_SetString(PyExc_StandardError, "Cannot connect to X Display");
		return NULL;
	}

	if(!XNVCTRLQueryAttribute(NVInfoDisplay, 0, 0,
                         NV_CTRL_GPU_CORE_TEMPERATURE, &Temp)) {
		PyErr_SetString(PyExc_StandardError, "Cannot query attribute");
		return NULL;
	}
	
	return Py_BuildValue("i", Temp);
}

PyObject *Get_Freq(PyObject *pSelf, PyObject *pArgs)
{
	PyObject *pRetVal, *pGPUClk, *pMEMClk;
	struct {
		int mem : 16;
		int gpu : 16;
	} Freq;

	if (!NVInfoDisplay) {
		PyErr_SetString(PyExc_StandardError, "Cannot connect to X Display");
		return NULL;
	}

	if(!XNVCTRLQueryAttribute(NVInfoDisplay, 0, 0,
                   NV_CTRL_GPU_CURRENT_CLOCK_FREQS, (int *)&Freq)) {
		PyErr_SetString(PyExc_StandardError, "Cannot query attribute");
		return NULL;
	}
	
	pGPUClk = PyInt_FromLong(Freq.gpu);
	pMEMClk = PyInt_FromLong(Freq.mem);

	pRetVal = PyTuple_New(2);
	PyTuple_SetItem(pRetVal, 0, pGPUClk);
	PyTuple_SetItem(pRetVal, 1, pMEMClk);

	return pRetVal;
}

void initNVInfo(void)
{
	int major, minor;
	PyObject *pModule;

	pModule = Py_InitModule("NVInfo", NVInfoMethods);
	
	if (!(NVInfoDisplay = XOpenDisplay(NULL)))
		return;

	if (!XNVCTRLQueryVersion(NVInfoDisplay, &major, &minor)) {
		NVInfoDisplay = NULL;
		return;
	}
}
